@@ -3,6 +3,10 @@ import time
 import json
 from uuid import uuid4
 import cgi
+import re
+import logging
+import Cookie
+
 
 ###
 ### Request handling code
@@ -14,6 +18,11 @@ def parse_netstring(ns):
     assert rest[len] == ',', "Netstring did not end in ','"
     return rest[:len], rest[len+1:]
 
+def to_bytes(data, enc='utf8'):
+    """Convert anything to bytes
+    """
+    return data.encode(enc) if isinstance(data, unicode) else bytes(data)
+
 
 class Request(object):
 
@@ -66,22 +75,36 @@ class Request(object):
     def version(self):
         return self.headers.get('VERSION')
 
+    @property
+    def cookies(self):
+        """Lazy generation of cookies from request headers."""
+        if not hasattr(self, "_cookies"):
+            self._cookies = Cookie.SimpleCookie()
+            if "cookie" in self.headers:
+                try:
+                    cookies = self.headers['cookie']
+                    self._cookies.load(to_bytes(cookies))
+                except Exception, e:
+                    logging.error('Failed to load cookies')
+                    self.clear_all_cookies()
+        return self._cookies
+
     @staticmethod
     def parse_msg(msg):
         """Static method for constructing a Request instance out of a
-        message read straight of a zmq socket.
+        message read straight off a zmq socket.
         """
         sender, conn_id, path, rest = msg.split(' ', 3)
         headers, rest = parse_netstring(rest)
         body, _ = parse_netstring(rest)
-
         headers = json.loads(headers)
 
         return Request(sender, conn_id, path, headers, body)
 
     def is_disconnect(self):
         if self.headers.get('METHOD') == 'JSON':
-            return self.data['type'] == 'disconnect'
+            logging.error('DISONNECT')
+            return self.headers.get('type') == 'disconnect'
 
     def should_close(self):
         """Determines if Request data matches criteria for closing request"""
@@ -92,9 +115,33 @@ class Request(object):
         else:
             return False
 
+    def get_arguments(self, name, strip=True):
+        """Returns a list of the arguments with the given name. If the argument
+        is not present, returns an empty list. The returned values are always
+        unicode.
+        """
+        values = self.arguments.get(name, [])
+        # Get rid of any weird control chars
+        values = [re.sub(r"[\x00-\x08\x0e-\x1f]", " ", x) for x in values]
+        values = [unicode(x) for x in values]
+        if strip:
+            values = [x.strip() for x in values]
+        return values
+
+    def get_argument(self, name, default=None, strip=True):
+        """Returns the value of the argument with the given name.
+
+        If the argument appears in the url more than once, we return the
+        last value.
+        """
+        args = self.get_arguments(name, strip=strip)
+        if not args:
+            return default
+        return args[-1]
+
 
 ###
-### Http handling code
+### Mongrel2 handling code
 ###
 
 CTX = zmq.Context()
@@ -140,14 +187,11 @@ class Mongrel2Connection(object):
         internally.
         """
         header = "%s %d:%s," % (uuid, len(str(conn_id)), str(conn_id))
-        #self.out_sock.send_pyobj(header + ' ' + msg)
-        #self.out_sock.send(header + ' ' + msg)
         self.out_sock.send_unicode(header + ' ' + msg)        
 
     def reply(self, req, msg):
         """Does a reply based on the given Request object and message.
         """
-
         self.send(req.sender, req.conn_id, msg)
 
     def reply_bulk(self, uuid, idents, data):
