@@ -14,8 +14,6 @@ See github.com/j2labs/brubeck for more information.
 import eventlet
 from eventlet import spawn, spawn_n, serve
 from eventlet.green import zmq
-from eventlet.hubs import get_hub, use_hub
-use_hub('zeromq')
 
 from . import version
 
@@ -338,8 +336,11 @@ class WebMessageHandler(MessageHandler):
     def body(self):
         return self._payload[self._BODY]
 
-    def set_body(self, body, headers=None):
+    def set_body(self, body, status_code=self._SUCCESS_CODE, headers=None):
+        """
+        """
         self._payload[self._BODY] = body
+        self.set_status(status_code)
         if headers is not None:
             self._payload[self._HEADERS] = headers
 
